// Casse Brique sur ecran Nokia 6100 en mode 256 couleurs
// CPU : PIC18F4680 , 40Mhz (10Mhz PLL x4)
//
// Muller Fabrice
// 10 Juin 2007
//

#include <LCD_6100_18F4680_KasBrik.h>
#include <stdlib.h>
#include <string.h>
#include <pcf8833.c>

// Declaration des Variables Globales

// Tableau de briques
unsigned char Brique[13][10];        // 10 lignes de 13 briques (ne pas utiliser int1 !!!)
// Position de la raquette
unsigned char Raquette,OldRaq;       // Position de depart de la raquette
unsigned char RaqDroite;             // Pixel droit de la raquette
unsigned char TailleRaq = 40;        // Taille de la raquette
// Position de la balle
unsigned char Ballex,Balley;         // Position de la balle
unsigned char OldBalleX,OldBalleY;   // Position de la balle sauvegard
// Direction de la Balle
signed int    Dx;                    // Direction X de la balle
signed int    Dy;                    // Direction Y de la balle
// Coordonne de la brique Touch
unsigned char   Touchex;
unsigned char   Touchey;
// Variables diverses
char   chiffre[5];                     // Pour Afficher un score (si necessaire)
char   gameover[] = "GAME OVER";       // Pas besoin d'explication ;)
char   Gagne[] = "On Recommence";      // Pas besoin d'explication ;)
int16 Score = 0;                       // ...

// Declaration des fonctions

void KasBrick(void);
void AfficheBrique(void);
void TestBrique(void);
void testraquette(void);
void GameOverTxt(void);
void Gagne_Partie(void);

void main()
{
   setup_adc_ports(AN0_TO_AN1|VSS_VDD);
   setup_adc(ADC_CLOCK_INTERNAL);
   setup_psp(PSP_DISABLED);
   setup_spi(FALSE);
   setup_wdt(WDT_OFF);
   setup_timer_0(RTCC_INTERNAL);
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DISABLED,0,1);
   setup_timer_3(T3_DISABLED|T3_DIV_BY_1);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   setup_oscillator(False);

   // L'ecran est sur le Port C
   set_tris_c(0x00);          // Port C en sorties
   output_c(0x00);            // tout le port C  0

   // Attendre que l'alimentation soit stabilise

   delay_ms(1000);

   // Toujour initialiser l'ecran en premier
   glcd_init(65);

   // Effacer l'ecran
   glcd_cls(BLACK);

   // Definir l'entre ADC  utiliser pour la raquette
   set_adc_channel( 0 );
   // adcresult  = Read_ADC();

   // Lancer le Jeux
   KasBrick();
}

void KasBrick(void)
{
   int16   value1 = 0;

   // Afficher les Briques
   affichebrique();

   // Initialiser Position Raquette
   Raquette = 0;

   // Initialiser Position de la Balle
   Ballex = 66 ;
   Balley = 120 ;
   OldBallex = Ballex ;
   OldBalley = Balley ;

   Dx = 1;
   Dy = -1;
   // Routine de traitement
   do
     {
      // Lire la valeur ADC
      value1 = read_adc();
      OldRaq = Raquette;
      Raquette = (unsigned char)(value1  * .128);
      // Tester sortie ecran de la raquette
      if (Raquette > (131 - TailleRaq)) Raquette = (131-TailleRaq);

      RaqDroite = Raquette + TailleRaq;
      // Effacer Ancienne Position de la Raquette si necessaire
      //       Et reafficher a la nouvelle Position
      if (Raquette != OldRaq)
       {
         glcd_rect(OldRaq,128,OldRaq + 40,131,1,YES,BLACK);
         glcd_rect(Raquette,128,RaqDroite,131,1,YES,BRIGHTBLUE);
       }
      OldBalleX = Ballex ;
      OldBalleY = Balley ;
      BalleX += Dx;
      BalleY += Dy;
      // La balle sort du Haut de l'ecran ?
      if (Balley < 3) Dy = abs(Dy);
      // La Balle arrive a Hauteur de la raquette en Y ?
      // Faire un Test pour voir si Balle receptionne
      if (Balley >= 127) TestRaquette();
      // La balle touche le bord droit de l'ecran ?
      if (Ballex > 128) Dx = -Dx;
      // La balle touche le bord gauche de d'ecran ?
      if (Ballex < 3) Dx = abs(Dx);
      Touchey = (unsigned char)(Balley / 6);
      glcd_rect(OldBallex - 1,Oldballey - 1,Oldballex + 1,OldBalley + 1,1,YES,BLACK);
      glcd_rect(Ballex - 1,balley - 1,ballex + 1,Balley + 1,1,YES,YELLOW);
      // La balle entre dans le Mur de brique ?
      if ((Touchey >= 0) && (Touchey <= 9))
         {
            Touchex = (unsigned char)(Ballex / 10);
            if (Touchex <=12)
               // Tester si touche le mur de briques
               TestBrique();
         }

      Delay_ms(20);
     }
   while (TRUE);

}

// Afficher les Briques en Debut de partie
void affichebrique(void)
 {
   int8  x;
   int8  y;

   for (x=0;x<13;x++)
      for (y=0;y<10;y++)
         {
          glcd_rect(x * 10, y * 6, (x*10)+8, (y*6)+4, 1, YES, BRIGHTGREEN);
          Brique[x][y] = 1;
         }
 }

// Tester si la Balle Touche une brique
void TestBrique(void)
 {
     // S'il y a une Brique
     if (Brique[Touchex][Touchey] == 1)
        {
          Brique[Touchex][Touchey] = 0;
          // Effacer la brique
          glcd_rect(Touchex * 10,Touchey * 6,(Touchex * 10) + 8,(Touchey * 6) + 4,1,YES,BLACK);
          // On augmente le score ...
          Score += 10;
          itoa(Score, 10, chiffre);
          glcd_rect(0,70,30,77,1,YES,BLACK);
          glcd_text57(0,70,chiffre,1,GREEN,BLACK);
          // On as detruit tout le mur ?? (Score Max = 1300)
          if (Score >= 1300) Gagne_Partie();
          // Verifier Direction en Y de la Balle et l'inverser
          if (Dy > 0)
            Dy = -Dy;
          else
            Dy = abs(Dy);
        }
 }

// Test pour voir si la Balle est pass derriere la raquette
void testraquette(void)
 {
   // balle a Droite de la raquette ?
   if (ballex < raquette)
      gameovertxt();
   // balle a Gauche de la raquette ?
   if (ballex > RaqDroite)
      gameovertxt();
   // balle sur la raquette on inverse Direction Y
   Dy = -Dy;
   // Maintenant on test ou la balle se rouve sur la raquette
   // Sur la partie Gauche de la raquette (bord)
   if ((ballex >= raquette) && (ballex <= (unsigned char)(raqdroite/4)))
      // Si la Balle vient de la Gauche on inverse Dx
      if (Dx > 0)
        Dx = -2;
   // Sur la partie Gauche de la raquette (1/4 vers centre)
   if ((ballex >= (unsigned char)(raqdroite/4)) && (ballex <= (unsigned char)(raqdroite/2)))
      // Si la Balle vient de la Gauche on inverse Dx
      if (Dx > 0)
        Dx = -1;

   // Sur la partie Droite de la raquette bord)
   if ((ballex >= ((unsigned char)(raqdroite / 2)+(unsigned char)(raqdroite / 4))) && (ballex <= raqdroite))
      // Si la balle vient de la Droite on inverse Dx
      if (Dx < 0)
        Dx = 2;
    // Sur la partie Droite de la raquette (1/4 aprs milieu vers droite)
   if ((ballex >= (unsigned char)(raqdroite / 2)) && ((ballex <= raqdroite)-(unsigned char)(raqdroite / 4)))
      // Si la balle vient de la Droite on inverse Dx
      if (Dx < 0)
        Dx = 1;
}

// Fin du jeux et Reset du Pic
void gameovertxt(void)
 {
   // La balle est pass derriere la raquette ...
   glcd_rect(20,25,110,40,1,YES,BLACK);
   glcd_text57(20,25,gameover,2,BRIGHTBLUE,BLACK);
   delay_ms(2000);
   // Effacer ecran
   glcd_CLS(BLACK);
   delay_ms(25);
   reset_cpu();
 }

void Gagne_Partie(void)
 {
   // La partie est gagne
   glcd_rect(20,25,131,40,1,YES,BLACK);
   glcd_text57(20,25,Gagne,2,BRIGHTBLUE,BLACK);
   delay_ms(2000);
   // Effacer ecran
   glcd_CLS(BLACK);
   delay_ms(25);
   reset_cpu();
 }

